import { onMounted, ref, nextTick } from "vue";
import { MarkerClusterer } from "@googlemaps/markerclusterer";

function waitForGmapLoaded() {
  return new Promise((resolve) => {
    let intervalId = setInterval(() => {
      if (window._gmapLoaded && window.google) {
        clearInterval(intervalId);
        resolve(true);
      }
    }, 150);
  });
}

export function useAutocomplete({ inputRef, options, onPlaceChanged }) {
  let autocompleteInstance = null;
  onMounted(() => {
    let inputElement = null;
    if (inputRef.value instanceof HTMLElement) {
      inputElement = inputRef.value;
    } else {
      inputElement = inputRef.value.$el.querySelector("input[type='text']");
    }

    autocompleteInstance = new google.maps.places.Autocomplete(inputElement, options);

    if (onPlaceChanged) {
      autocompleteInstance.addListener('place_changed', () => onPlaceChanged(autocompleteInstance));
    }
  });

  return { autocompleteInstance };
}

export function useGmap({ mapRef, infoWindowRef, options, locations, showCluster = false, showDirection = false }) {
  let mapInstance = null;
  const markers = ref([]);
  const cluster = ref({});
  const selectedMarker = ref({});

  let infoWindow = null;

  const drawDirection = () => {
    let directionService = new google.maps.DirectionsService();
    let directionDisplay = new google.maps.DirectionsRenderer();

    for (let l = 0; l < locations.length; l++) {
      for (let i = 0; i < locations[l].length; i++) {
        let travel = {
          origin: locations[l][i],
          destination: locations[l][i + 1],
          travelMode: "DRIVING"
        };

        if (!travel.destination) {
          break;
        }

        directionService.route(travel, (response, status) => {
          if (status === "OK") {
            directionDisplay = new google.maps.DirectionsRenderer({
              suppressMarkers: true,
              polylineOptions: {
                strokeColor: "#2ab3a6",
                stropeOpacity: 0.8,
                strokeWeight: 5
              }
            });
            directionDisplay.setMap(mapInstance);
            directionDisplay.setDirections(response);
          }
        });
      }
    }
  };

  const updateMarkers = (locations) => {
    // remove old markers
    if (showCluster && cluster.value.length >= 1) {
      cluster.value.clearMarkers();
    }

    if (markers.value && markers.value.length >= 1) {
      markers.value.forEach((marker) => marker.setMap(null));
    }

    // show the new one
    if (locations && locations.length >= 1) {
      markers.value = locations.map((value) => {
        const marker = new google.maps.Marker({
          position: value,
          map: mapInstance,
          icon: value.imageUrl
        });

        if (value.infoWindow) {
          marker.addListener('click', () => {
            selectedMarker.value = value;

            nextTick(() => {
              infoWindow.setContent(infoWindowRef.value);

              infoWindow.open({
                map: mapInstance,
                anchor: marker
              });
            });

          });
        }

        return marker;
      });
    }

    if (showCluster) {
      cluster.value = new MarkerClusterer({ map: mapInstance, markers: markers.value });
    }
  };

  const mapPromise = () => {
    return new Promise((resolve) => {
      let intervalId = setInterval(() => {
        if (mapInstance) {
          clearInterval(intervalId);
          resolve(mapInstance);
        }
      }, 50);
    });
  };

  onMounted(async () => {
    await waitForGmapLoaded();
    // initialize static info window (only one info window per one map)
    infoWindow = new google.maps.InfoWindow;

    const defaultOptions = {
      zoom: 4,
      center: { lat: -42.162376, lng: 172.671437 },
      mapId: Math.random().toString(36).slice(2, 7)
    };

    mapInstance = new google.maps.Map(mapRef.value, { ...defaultOptions, ...options });

    if (locations) {
      updateMarkers([locations].flat());
    }
  });

  return { updateMarkers, mapPromise, mapInstance, selectedMarker, drawDirection };
}



