import Action from "../../action";
import UrlUtils from "../../utils/url";
import TypeUtils from "../../utils/type";
import { vueApp } from "../../store";

export default {
  data() {
    return {
      // TODO: Consider replacing with an enum, e.g. executing with predefined values ('post', 'get', null)
      // The complexity with this is that 'get' that comes after 'post' needs to show an indicator, e.g.
      // Submit button needs to be disabled when redirecting after form submission
      _isBusy: false,
      _events: [],
      _typingTimer: null,
      _mountedUrl: null,
    };
  },
  computed: {
    _pageBody() {
      return document.getElementById("page_body");
    },
    $isBusy() {
      // Properties starting with $ or _ can only be accessed via $data
      return this.$data._isBusy;
    },
    $isReady() {
      return this.$data._events === null;
    }
  },
  created() {
    // watch components/update
    if (this.spec) {
      this.$watch(
        () => Object.keys(this.spec).map((specKey) => this.spec[specKey]),
        () => this._ready()
      );
    }

    this.$created();
  },
  mounted() {
    // this._executeIfReady(false);
    this._ready();
    this.$mounted();
  },
  beforeUpdate() {
    if (vueApp.isStale) {
      this._mountedUrl = null;
    }

    if (!this._renderingTheSamePage()) {
      this.$tearDown();
      this._ready();
    }
  },
  updated() {
    // this._executeIfReady(true);
    this.$updated();
  },
  // deactivated() {
  unmounted() {
    this.$tearDown();
  },
  methods: {
    $href(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;

      if (TypeUtils.isObject(onClick)) {
        switch (onClick.action) {
          case "windows/open":
          case "windows/open-v1":
          case "windows/openWeb":
          case "windows/openWeb-v1":
          case "windows/reload":
          case "windows/reload-v1":
            return UrlUtils.htmlUrl(onClick.url);
        }
      }
      return '';
    },
    $onClick: function (explicitEvent, spec) {
      const properties = spec || this.spec;
      const e = explicitEvent || event;

      if (!properties.onClick) {
        return; // Avoid blocking the event for parents.
      }

      // Ignore middle/right clicks
      if (!e.metaKey && !e.ctrlKey && e.button === 0) {
        e.preventDefault();
        e.stopPropagation();
        Action.execute(properties.onClick, this);
      }
    },
    $rel: function (spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        if (onClick.action === "windows/openWeb-v1") {
          return "nofollow";
        }
      }
      return null;
    },
    // Can be used for optimization, but not needed for now.
    // $onLongPressFunction() {
    //   return this.spec.onLongPress ? this.$onLongPress : null;
    // },
    $onLongPress() {
      GLib.action.execute(this.spec.onLongPress, this);
    },
    $addViewportChangeListeners: function (handler) {
      window.addEventListener("load", handler);
      window.addEventListener("resize", handler);
      this._pageBody.addEventListener("scroll", handler);
    },
    $removeViewportChangeListeners: function (handler) {
      window.removeEventListener("load", handler);
      window.removeEventListener("resize", handler);
      this._pageBody.removeEventListener("scroll", handler);
    },
    _renderingTheSamePage() {
      return this._mountedUrl === window.location.href;
    },
    _ready() {
      this._mountedUrl = window.location.href;
      // this._updatedAt = new Date().getTime();

      this.$data._events = [];

      this.$ready();

      // Make sure events are dispatched after $ready(). Dispatching them during $ready() causes errors.
      for (const event of this.$data._events) {
        this.$el.dispatchEvent(event);
      }
      this.$data._events = null;
    },
    // Order of execution:
    // Initial page load: $created() -> $ready() -> $mounted() -> $updated() (because of $ready(), may get called multiple times)
    // windows_open: $created() (only if it isn't reused) -> $ready() -> $mounted() (only if it isn't reused) -> $updated()
    // components_update: $ready() -> $updated() (because of $ready(), may get called multiple times)
    $ready() {
      // To be overridden. Initialize component data here.
      // This method has to be idempotent because it might get called multiple times (due to components_update).
    },
    $created() {
      // To be overridden
    },
    $tearDown() {
      // To be overridden
    },
    $mounted() {
      // To be overridden. Put one-off event listener initialization here (e.g. $onEvent()).
    },
    $updated() {
      // To be overridden
    },
    // Override to provide customization
    $update() {
      this.$ready();
    },
    async $recursiveUpdate(stack) {
      // const children = this.$children;
      if (this.spec) {
        // this.$update();
        // this.$forceUpdate();

        // Execute on next tick to ensure that the child has received the updated spec.
        // $children is removed in vue 3
        // this.$nextTick(() => {
        //   this.$children.find(child => {
        //     child.$recursiveUpdate();
        //   });
        // });
      }
    },
    $dispatchEvent(name, data) {
      const event = new Event(name, { bubbles: true });

      // TODO: Deprecate
      event.spec = data;

      event.data = data;

      if (this.$data._events) {
        this.$data._events.push(event);
      } else {
        this.$el.dispatchEvent(event);
      }
    },
    $onEvent(name, handler) {
      this.$el.addEventListener(name, handler);
    },
    $onTyping({ duration = 2000 }) {
      GLib.action.execute(this.spec.onTypeStart, this);
      clearTimeout(this.timer);

      this.timer = setTimeout(() => {
        this._typingTimer = null;
        GLib.action.execute(this.spec.onTypeEnd, this);
      }, duration);
    },
    $executeOnChange(newValue) {
      Utils.type.ifObject(this.spec.onChange, onChange => {
        this.$nextTick(() => {
          const value = newValue || this.fieldModel;

          const formData = {
            [this.fieldName]: value
          };
          const params = this.spec.paramNameForFormData
            ? {
              [this.spec.paramNameForFormData]: formData
            }
            : formData;
          const data = Object.assign({}, onChange, { formData: params });
          GLib.action.execute(data, this);
        });
      });
    }
  }
};
