import Hash from "../../utils/hash";
import { fieldModels, watchFieldModels } from "../composable/conditional";
import { useDirtyState } from "../composable/dirtyState";
const { updateDirtyState } = useDirtyState();

export default {
  data: function () {
    return {
      fieldName: null,
      fieldModel: null,
      _show: true,
      _watchers: [],
      _isValueChanged: false,
      // Some components do not support null or empty string value, so we need to use an intermediary value.
      // See https://github.com/vuetifyjs/vuetify/issues/8876
      vuetifyEmptyString: "<EMPTY_STRING>"
    };
  },
  provide() {
    const obj = {};
    const name = GLib.component.vueName(this);
    const shareContextIf = name && this.spec && name.startsWith('panels') && this.spec.loadIf;
    if (shareContextIf) {
      obj['panelContext'] = this.$data;
    }

    return obj;
  },
  inject: {
    panelContext: { default: {} }
  },
  computed: {
    inputDisabled() {
      let disabled = false;
      if (Object.keys(this.panelContext).length > 0) { // handle nested
        disabled = !this.panelContext._show;
      } else if (this.spec.loadIf) { // handle direct
        disabled = !this._show;
      }

      return disabled;
    }
  },
  watch: {
    fieldModel: function (val, oldVal) {
      if (val === this.vuetifyEmptyString) {
        val = "";
      }

      Object.assign(fieldModels, { [this.fieldName]: this.$internalizeValue(val) });

      updateDirtyState(val, oldVal, this.spec);
    },
    spec: {
      handler(spec, oldSpec) {
        if (spec) {
          let valueChanged = true;
          if (oldSpec && oldSpec.value === spec.value && oldSpec.name === spec.name) {
            valueChanged = false;
          }
          this._linkFieldModels(valueChanged);
          this._initConditional(oldSpec);
        }
      },
      immediate: true
    }
  },
  unmounted() {
    if (fieldModels[this.fieldName]) {
      delete fieldModels[this.fieldName];
    }
  },
  methods: {
    _initConditional(oldSpec) {
      // unreg old fieldName
      if (oldSpec && fieldModels[oldSpec.name] && oldSpec.name != this.spec.name) {
        delete fieldModels[oldSpec.name];
      }
      // unreg watchers if there is
      this._watchers.forEach(watcher => {
        if (typeof watcher == Function) {
          watcher();
        }
      });

      let watcher1, watcher2;

      //  watch showIf & loadIf
      if (this.spec && this.spec.valueIf) {
        watcher1 = watchFieldModels(this.spec.valueIf, (value) => {
          // prevent override spec.value
          if (!this._isValueChanged && this.spec.value) {
            this._isValueChanged = true;
            return;
          }

          this.fieldModel = this.$sanitizeValue(this.$externalizeValue(value));
        });
        this._watchers.push(watcher1);
      }

      if (this.spec && (this.spec.showIf || this.spec.loadIf)) {
        watcher2 = watchFieldModels(this.spec.showIf || this.spec.loadIf, (value) => {
          if (value) {
            this._show = true;
          } else {
            this._show = false;
          }
        });
        this._watchers.push(watcher2);
      }
    },
    // TODO: Deprecated
    genericStyles(spec) {
      return this.$styles(spec);
    },
    setFieldModel(name, value) {
      fieldModels[name] = value;
    },
    getFieldModel(name) {
      return fieldModels[name];
    },
    // _checkDirtyState(val, oldVal) {
    //   const dirtyCheckEnabled = !this.spec.disableDirtyCheck;
    //   // Make sure value has changed and make sure that it is different from the original value.
    //   // Be strict with this so it doesn't execute when the component is just initializing (e.g value changing
    //   // from `null` to `this.spec.value`).
    //   if (dirtyCheckEnabled && val != oldVal && val != this.spec.value) {
    //     const topDialog = Utils.launch.dialog.topDialog();
    //     if (topDialog) {
    //       if (!topDialog.isFormDirty) {
    //         console.debug("Dialog form is now dirty");
    //         topDialog.isFormDirty = true;
    //       }
    //     } else {
    //       if (vueApp.isFormDirty) {
    //         console.log("Window form is now dirty");
    //         vueApp.isFormDirty = true;
    //       }
    //     }
    //   }
    // },
    // NOTE: Styles are dynamic, do not save it in $ready().
    $styles(spec) {
      const properties = spec || this.spec;
      const styles = {
        // color: properties.color,
        // "background-color": properties.backgroundColor,
        // "text-align": properties.textAlign
      };

      Utils.type.ifString(properties.color, val => (styles["color"] = val));

      Utils.type.ifString(
        properties.backgroundColor,
        val => (styles["background-color"] = val)
      );

      Utils.type.ifString(
        properties.textAlign,
        val => (styles["text-align"] = val)
      );

      Utils.type.ifObject(properties.padding, padding => {
        Utils.type.ifNumber(
          padding.top || padding.y || padding.all,
          top => (styles["padding-top"] = `${top}px`)
        );
        Utils.type.ifNumber(
          padding.bottom || padding.y || padding.all,
          bottom => (styles["padding-bottom"] = `${bottom}px`)
        );
        Utils.type.ifNumber(
          padding.left || padding.x || padding.all,
          left => (styles["padding-left"] = `${left}px`)
        );
        Utils.type.ifNumber(
          padding.right || padding.x || padding.all,
          right => (styles["padding-right"] = `${right}px`)
        );
      });

      Utils.type.ifString(
        this.$length(properties.width),
        val => (styles["width"] = val)
      );
      Utils.type.ifString(
        this.$length(properties.height),
        val => (styles["height"] = val)
      );

      if (this._show) {
        styles['display'] = this.$displayValue();
      } else {
        styles['display'] = 'none';
      }

      return new Hash(styles);
    },
    $length: function (length) {
      if (Utils.type.isString(length) || Utils.type.isNumber(length)) {
        switch (length) {
          case "wrapContent":
            return null;
          case "matchParent":
            return "100%";
          default:
            return `${length}px`;
        }
      }
      return null;
    },
    // $builtinColor(styleClasses) {
    //   let color = null;
    //   Utils.type.ifArray(styleClasses, classes => {
    //     for (const val of ["success", "info", "warning", "error"]) {
    //       if (classes.remove(val)) {
    //         color = val;
    //       }
    //     }
    //   });
    //   return color;
    // },
    $displayValue() {
      // return "block"; Deprecated because it causes problems for multiple rows components
      // return "inherit"; // To be overridden

      // Use null so we don't accidentally override existing styling. Without this, the
      // `_read_more_text` garage example will not work.
      return null;
    },
    _sanitizeValue(val) {
      return Utils.type.isNotNull(val) ? this.$sanitizeValue(val) : null;
    },
    $sanitizeValue(val) {
      return val; // To be overridden
    },
    $internalizeValue(val) {
      return val;
    },
    $externalizeValue(val) {
      return val;
    },
    _linkFieldModels(valueChanged) {
      const hasCondition = this.spec && this.spec.showIf;
      const name = GLib.component.vueName(this);

      const isField = name && name.startsWith("fields-");
      if (hasCondition || isField) {
        // Has to be executed before $ready(). This executes regardless of whether a form is found because fields
        // may be used without a form.
        this.fieldName = this.spec.name;

        // Avoid user input getting removed when updating certain parts of the page.
        if (valueChanged) {
          this.fieldModel = this._sanitizeValue(this.spec.value);
        }
      }
    },
    action_resetValue() {
      this.fieldModel = this._sanitizeValue(this.spec.value);
    },
    $classes(spec, defaultViewName) {
      const properties = Object.assign(
        { view: defaultViewName },
        spec || this.spec
      );
      const componentName = Utils.app.componentName(properties.view);
      const classes = (properties.styleClasses || []).concat(componentName);

      Utils.type.ifObject(properties.onClick, () =>
        classes.push("glib-clickable")
      );

      if (properties.readOnly) {
        classes.push("glib-input-disabled");
      }

      // This could appear not as before after migration to vuetify 3
      Utils.type.ifArray(properties.styleClasses, val => {
        if (val.includes("card")) {
          classes.push("v-card");
          classes.push("v-sheet");
          classes.push("theme--light");

          if (val.includes("outlined")) {
            classes.push("v-sheet--outlined");
          }
        }
      });

      return classes;
    }
  }
};
