import Type from "./type";
import Action from "../action";
import { nextTick } from 'vue';
import { ctx, vueApp } from "../store";

let loading = false;

class HttpRequest {
  constructor() {
    this.controller = new AbortController();
    this.canceled = false;
  }

  cancel() {
    this.canceled = true;
    this.controller.abort();
  }
}

export default class {
  static _extractFormData(properties, needCsrfToken) {
    // if (properties["formDataEncoded"]) {
    //   return properties["formData"];
    // }

    const data = properties.formData;
    if (data instanceof FormData) {
      return data;
    }

    const formData = new FormData();
    var params = data || {};
    for (const key in params) {
      // formData.append(key, params[key]);
      this._populateFormData(formData, key, params[key]);
    }

    if (needCsrfToken && !Type.isString(params["authenticity_token"])) {
      formData.append("authenticity_token", Utils.dom.getCsrf());
    }
    return formData;
  }

  static _populateFormData(formData, key, value) {
    if (GLib.type.isArray(value)) {
      for (const item of value) {
        this._populateFormData(formData, `${key}[]`, item);
      }
    } else if (GLib.type.isObject(value)) {
      for (const innerKey in value) {
        this._populateFormData(
          formData,
          `${key}[${innerKey}]`,
          value[innerKey]
        );
      }
    } else {
      formData.append(key, value);
    }
  }

  static load(properties, component, windowMode) {
    const urlString = properties["url"];
    let url;
    try {
      url = new URL(urlString);
    } catch (e) {
      console.warn("Invalid URL", urlString);
      throw e;
    }
    const domainMatched = window.location.hostname == url.hostname;

    // If this is an external domain, we rely on `onUnload()` instead.
    // if (domainMatched && !this.proceedEvenWhenDirty()) {
    //   return;
    // }

    if (Utils.settings.reactive && domainMatched) {
      const currentUrl = window.location.href;
      const htmlUrl = Utils.url.htmlUrl(properties["url"]);

      Utils.http.execute(properties, "GET", component, (data, response) => {
        const pushHistory = windowMode ? true : !Utils.type.isObject(component.$closest("dialog"))
        if (htmlUrl !== currentUrl && pushHistory) {
          const redirectUrl = Utils.url.htmlUrl(response.url);
          Utils.history.pushPage(data, redirectUrl);
        }

        this.forceComponentUpdate(() => {
          Utils.history.resetScroll();

          GLib.action.handleResponse(data, component, windowMode)

          Action.execute(properties["onOpen"], component);
        });
      });
    } else {
      window.location = Utils.url.htmlUrl(properties["url"]);
    }
    // bus.$emit("glibWebHttpLoad", new URL(Utils.url.htmlUrl(properties["url"])));
  }

  static analyticsHeaders(component) {
    const pageAnalytics =
      vueApp.temp.analytics || vueApp.page || {};
    const featureAnalytics = pageAnalytics.feature;

    const headers = {};
    if (featureAnalytics && component.spec) {
      const componentAnalytics = component.spec.analytics || {};
      if (componentAnalytics.disabled) {
        return headers;
      }

      Utils.type.ifString(
        componentAnalytics.group || featureAnalytics.group,
        value => (headers["GApp-Analytics-Referer-Group"] = value)
      );

      Utils.type.ifString(
        componentAnalytics.action || featureAnalytics.action,
        value => (headers["GApp-Analytics-Referer-Action"] = value)
      );

      Utils.type.ifString(
        componentAnalytics.segment || featureAnalytics.segment,
        value => (headers["GApp-Analytics-Referer-Segment"] = value)
      );

      Utils.type.ifString(
        componentAnalytics.placement,
        value => (headers["GApp-Analytics-Referer-Placement"] = value)
      );
    }
    return headers;
  }

  static reload(properties, component) {
    const currentUrl = window.location.href;
    const data = {
      url: properties.url || currentUrl,
      formData: properties.formData
    };

    Utils.http.execute(data, "GET", component, (page, response) => {
      Utils.http.forceComponentUpdate(() => {
        vueApp.page = page;
        const redirectUrl = Utils.url.htmlUrl(response.url);
        Utils.history.updatePage(vueApp.page, redirectUrl);

        GLib.action.execute(properties["onReload"], component);
      });
    });
  }

  static execute(properties, methodName, component, jsonHandler, errorHandler) {
    this.startIndicator(component);

    // `fetch()` only supports uppercase
    const method = methodName.toUpperCase();
    let url = Utils.url.jsonUrl(properties["url"]);
    let body = this._extractFormData(properties, method !== "GET");
    if (method === "GET") {
      url = Utils.url.appendParams(url, body);
      body = null;
    }
    const silent = properties.silent;
    if (!silent) {
      console.debug(`${method} ${url}`);
    }

    const request = new HttpRequest();
    let response = null;
    const vm = this;

    const analyticsHeaders = this.analyticsHeaders(component);
    const headers = Object.assign(
      { Accept: "text/html" },
      analyticsHeaders,
      GLib.settings.headerAugmenter(url, method)
    );

    fetch(url, {
      method: method,
      body: body,
      headers: headers,
      // Make sure to pass cookies for same origin URLs.
      // Needed for some browsers, e.g. Edge and Android's native.
      credentials: "same-origin",
      signal: request.controller.signal
    })
      .then(res => {
        vm.stopIndicator(component);

        response = res;
        if (res.status >= 500) {
          throw "Server error";
        } else if (res.status >= 400) {
          throw "Not accessible";
        } else {
          return res.json();
        }
      })
      .then(data => {
        if (!silent) {
          console.debug("Success", data);
        }
        jsonHandler(data, response);
      })
      .catch(error => {
        vm.stopIndicator(component);

        if (!request.canceled) {
          const message = error.toString();

          console.error("Error:", message);

          if (errorHandler) {
            errorHandler(error, response);
          } else {
            if (!silent) {
              Utils.launch.snackbar.error(message, component);
            }
          }
        } else {
          console.info("Canceled");
        }
      });

    return request;
  }

  static startIndicator(component) {
    this._showIndicator();
    component.$data._isBusy = true;
  }

  static stopIndicator(component) {
    this._hideIndicator();
    component.$data._isBusy = false;
  }

  static _showIndicator() {
    loading = true;
    setTimeout(() => {
      if (loading) {
        loading = false;
        vueApp.indicator = true;
      }
    }, 200);
  }

  static _hideIndicator() {
    loading = false;
    vueApp.indicator = false;
  }

  // See generic.js
  static forceComponentUpdate(handler) {
    // GLib.component.clearRegistry();
    vueApp.isStale = true;
    ctx().isFormSubmitted = false;
    ctx().isFormDirty = false;
    handler();

    // Queue the execution so the first isStale has time to resets state before handler gets executed
    nextTick(() => {
      vueApp.isStale = false;
    });
  }

  // static clearDirtyState() {
  //   vueApp.isFormDirty = false;
  // }

  static notifyFormSubmitted() {
    ctx().isFormSubmitted = true;
  }

  // `context` can be either window or dialog.
  // static proceedEvenWhenDirty() {
  //   const dirtyContext = vueApp;
  //   // Don't prompt if this is a result of form submission
  //   if (
  //     dirtyContext.isFormDirty &&
  //     !dirtyContext.isFormSubmitted &&
  //     !confirm(dirtyPrompt)
  //   ) {
  //     return false;
  //   }
  //   return true;
  // }
}
