import { Vue } from "../index";
import Dialog from "../nav/dialog.vue";
import Sheet from "../nav/sheet.vue";
import Snackbar from "../nav/snackbar.vue";
import Popover from "../components/popover.vue";
import { computePosition, flip, offset } from '@floating-ui/dom';
import bus from "./eventBus";
import { nextTick } from "vue";

import { createApp, h } from "vue";

export default class {
  static get dialog() {
    return LaunchDialog;
  }

  static get sheet() {
    return LaunchSheet;
  }

  static get snackbar() {
    return LaunchSnackbar;
  }

  static get popover() {
    return LaunchPopover;
  }
}

class LaunchPopover {
  static open(properties, component) {
    const instance = createApp(
      {
        render: function () {
          return h(Popover, { spec: properties });
        }
      }
    );
    Object.assign(instance._context, Vue._context);

    if (component) {
      const placeholder = document.createElement('div');

      // const pageBody = Utils.launch.dialog.closestBody(component) || Utils.history._pageBody;
      const dialogBody = Utils.launch.dialog.closestBody(component);
      if (dialogBody) {
        // Put the popover on the dialog so that clicking it doesn't close a non-persistent dialog.
        dialogBody.appendChild(placeholder);
      } else {
        // Attach the popover to the document body to prevent clipping caused by the parent's `overflow` property.
        // See https://stackoverflow.com/questions/51393427/z-index-to-ignore-max-height-and-always-come-on-top
        document.body.appendChild(placeholder);
      }

      instance.mount(placeholder);

      const reference = component.$el instanceof HTMLElement ? component.$el : component.$el.nextElementSibling;

      const floating = placeholder;
      const placement = properties.placement || 'right';
      const offsetSize = properties.offset || 8;

      // These properties need to be set to get an accurate calculation. See https://floating-ui.com/docs/computeposition
      Object.assign(placeholder.style, { position: 'absolute', display: 'block', top: 0, left: 0 });

      computePosition(reference, floating, { placement: placement, middleware: [flip(), offset(offsetSize)] })
        .then(({ x, y }) => {
          Object.assign(placeholder.style, { top: `${y}px`, left: `${x}px`, zIndex: 1200 });
        });
    } else {
      console.error("A popover has to be displayed in a component");
    }

    return instance;
  }

  static close(properties) {
    bus.$emit(`popover/close-${properties.key}`, properties);
  }
}

class LaunchDialog {
  static closestBody(component) {
    const element = component.$el instanceof HTMLElement ? component.$el : component.$el.nextElementSibling;
    const dialog = element.closest(".v-dialog");
    if (dialog) {
      return dialog.querySelector(".dialogs-body");
    }
    return null;
  }

  static _initStack() {
    if (!this.stack) {
      this.stack = [];
    }
  }

  static topDialog() {
    this._initStack();

    return this.stack.last();
  }

  static open(properties, component) {
    if (!this.stack) {
      this.stack = [];
    }

    // https://css-tricks.com/creating-vue-js-component-instances-programmatically/
    const props = {
      spec: properties,
      stack: this.stack
    };

    const instance = createApp(
      {
        render: function () {
          return h(Dialog, props);
        }
      }
    );
    Object.assign(instance._context, Vue._context);

    if (component) {
      const placeholder = document.createElement('div');
      document.body.appendChild(placeholder);
      instance.mount(placeholder);
    } else {
      console.error("A dialog has to be displayed in a component");
    }
  }

  static reload(properties, component) {
    if (!this.stack) {
      return;
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.reload(properties);
    });
  }

  static close(properties, component) {
    if (!this.stack) {
      this.stack = [];
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.close();
    });

    nextTick(() => {
      Action.execute(properties.onClose, component);
    })
  }

  // This is only meant to be used internally
  static alert(message, component) {
    const properties = {
      message: message
    };
    const spec = Object.assign({}, properties, {
      disableCloseButton: true,
      buttons: [
        {
          text: "OK",
          onClick: properties.onClose
        }
      ]
    });
    this.open(spec, component);
  }
}

class LaunchSheet {
  static open(properties, component) {
    const instance = createApp(
      {
        render: function () {
          return h(Sheet, { spec: properties });
        }
      }
    );
    Object.assign(instance._context, Vue._context);

    if (component) {
      const placeholder = document.createElement('div');
      document.body.appendChild(placeholder);
      instance.mount(placeholder);
    } else {
      console.error("A sheet has to be displayed in a component");
    }

    return instance;
  }

  static confirm(properties, onConfirm, component) {
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "Confirm",
          onClickFunction: onConfirm
        }
      ]
    });
    this.open(spec, component);
  }

  static alert(message, buttons, component) {
    const spec = { message: message, buttons: buttons };
    this.open(spec, component);
  }
}

class LaunchSnackbar {
  static open(properties, component) {
    const instance = createApp(
      {
        render: function () {
          return h(Snackbar, { spec: properties });
        }
      }
    );
    Object.assign(instance._context, Vue._context);

    if (component) {
      const placeholder = document.createElement('div');
      document.body.appendChild(placeholder);
      instance.mount(placeholder);
      instance.show = true;
    } else {
      console.error("A Snackbar has to be displayed in a component");
    }

    return instance;
  }

  static error(message, component) {
    this.open({ message: message, styleClasses: ["error"] }, component);
  }

  static indicator(message, component) {
    const instance = this.open({ message: message, timeout: -1 }, component);
    instance.indicator = true;
  }
}
